@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2017 New York University
+# Copyright (C) 2014 New York University
 # This file is part of ReproZip which is released under the Revised BSD License
 # See file LICENSE for full license details.
 
@@ -10,6 +10,7 @@ depending on what Linux distribution we are running on.
 
 Currently supported package managers:
 - dpkg (Debian, Ubuntu)
+- rpm (CentOS, Fedora)
 """
 
 from __future__ import division, print_function, unicode_literals
@@ -29,7 +30,7 @@ logger = logging.getLogger('reprozip')
 
 
 magic_dirs = ('/dev', '/proc', '/sys')
-system_dirs = ('/bin', '/etc', '/lib', '/sbin', '/usr', '/var')
+system_dirs = ('/bin', '/etc', '/lib', '/sbin', '/usr', '/var', '/run')
 
 
 class PkgManager(object):
@@ -132,8 +133,8 @@ class DpkgManager(PkgManager):
                                     stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE)
             out, err = proc.communicate()
-            for l in out.splitlines():
-                pkgname, path = l.split(b': ', 1)
+            for line in out.splitlines():
+                pkgname, path = line.split(b': ', 1)
                 path = Path(path.strip())
                 # 8-bit safe encoding, because this might be a localized error
                 # message (that we don't care about)
